﻿Imports System.Runtime.InteropServices

''' <summary>
''' Класс для работы LPT портом. При создании определяет наличие драйвера и его разрядность.
''' </summary>
''' <remarks>Copyright 2016 aave http://SolTau.ru</remarks>
Public NotInheritable Class LPT

#Region "DllImports"

    Private Const DLL32 As String = "inpout32.dll"
    Private Const DLL64 As String = "inpoutx64.dll"

    <DllImport(DLL32, CharSet:=CharSet.Auto, EntryPoint:="IsInpOutDriverOpen")> _
    Private Shared Function IsInpOutDriverOpen() As Byte
    End Function

    <DllImport(DLL64, CharSet:=CharSet.Auto, EntryPoint:="IsInpOutDriverOpen")> _
    Private Shared Function IsInpOutDriverOpen_x64() As UInt64
    End Function

    <DllImport(DLL32, CharSet:=CharSet.Auto, EntryPoint:="Inp32")> _
    Private Shared Function Inp32(ByVal PortAddress As UInt32) As Byte
    End Function


    <DllImport(DLL32, CharSet:=CharSet.Auto, EntryPoint:="Out32")> _
    Private Shared Sub Out32(ByVal portAddress As UInt32, ByVal data As UInt32)
    End Sub

    <DllImport(DLL64, CharSet:=CharSet.Auto, EntryPoint:="Inp32")> _
    Private Shared Function Inp64(ByVal PortAddress As UInt64) As Byte
    End Function

    <DllImport(DLL64, CharSet:=CharSet.Auto, EntryPoint:="Out32")> _
    Private Shared Sub Out64(ByVal PortAddress As UInt64, ByVal Data As UInt64)
    End Sub

#End Region

#Region "CTOR"

    Shared Sub New()
        If getIsDriverX86() Then
            _IsX64DriverUsed = False
        ElseIf getIsDriverX64() Then
            _IsX64DriverUsed = True
        Else
            Throw New SystemException("Драйвер для LPT порта не установлен.")
        End If
    End Sub

#End Region '/CTOR

#Region "PROPS"

    ''' <summary>
    ''' Используется ли 64-разрядная версия драйвера.
    ''' Если драйвер не установлен, возвращается NULL.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared ReadOnly Property IsX64DriverUsed As Boolean
        Get
            Return _IsX64DriverUsed
        End Get
    End Property
    Private Shared _IsX64DriverUsed As Boolean

#End Region '/PROPS

#Region "METHODS"

    ''' <summary>
    ''' Определяет, является ли драйвер для 32-разрядной версии Windows.
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Shared Function getIsDriverX86() As Boolean
        Try
            Dim res As Byte = IsInpOutDriverOpen()
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    ''' <summary>
    ''' Определяет, является ли драйвер для 64-разрядной версии Windows.
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Shared Function getIsDriverX64() As Boolean
        Try
            Dim nResult As ULong = IsInpOutDriverOpen_x64()
            If nResult <> 0 Then
                Return True
            Else
                Return False
            End If
        Catch ex As Exception
            Return False
        End Try
    End Function

#End Region '/METHODS

#Region "Read / Write LPT Port"

    ''' <summary>
    ''' Читает из LPT порта 1 байт данных.
    ''' </summary>
    ''' <param name="port">Номер порта.</param>
    ''' <param name="reg">Регистр порта.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function Read(ByVal port As ULong, Optional ByVal reg As Register = Register.DATA) As Byte

        Dim data As Byte = 0
        If IsX64DriverUsed Then
            data = Inp64(CULng(port + reg))
        Else
            data = Inp32(CUInt(port + reg))
        End If
        Return data

    End Function

    ''' <summary>
    ''' Записывает в LPT порт число.
    ''' </summary>
    ''' <param name="port">Адрес порта.</param>
    ''' <param name="reg">Регистр порта.</param>
    ''' <param name="data">Число для записи. В 32-разрядных системах аргумент должен быть не более, чем максимальное значение для типа UInt32.</param>
    ''' <remarks></remarks>
    Public Shared Sub Write(ByVal port As ULong, ByVal reg As Register, ByVal data As ULong)

        If IsX64DriverUsed Then
            Out64(port + CULng(reg), data)
        Else
            If data <= UInteger.MaxValue Then
                Out32(CUInt(port + reg), CUInt(data))
            Else
                Throw New ArgumentException("В 32-разрядных системах аргумент должен быть 32-разрядным (тип UInt32).", "data")
            End If
        End If

    End Sub

    ''' <summary>
    ''' Регистры порта LPT.
    ''' </summary>
    ''' <remarks></remarks>
    Public Enum Register As Integer
        DATA = 0    'регистр данных
        STATUS = 1  'статуса
        CONTROL = 2 'контроля
    End Enum

#End Region '/"Read / Write LPT Port

End Class